package yrpg.bamp.base.webapi;

import javax.validation.Valid;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.openapi.api.result.ApiSaveItemData;
import kd.bos.openapi.api.result.ApiSaveResult;
import kd.bos.openapi.api.result.BaseFilterItemData;
import kd.bos.openapi.api.result.BaseFilterResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.openapi.common.result.OpenApiResult;
import kd.bos.openapi.api.result.ApiSaveRowErrorData ;
import kd.bos.openapi.common.util.OpenApiSdkUtil;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@ApiController(value="CommonBill",desc="统一url入口")
public class AdminApiController {
	//存放实际地址和接口入口的
	@ApiPostMapping(value="adminURLApiHandler")
	public CustomApiResult<Object> adminApiHandler(@Valid  @ApiParam("入口参数") AdminApiReqModel reqModel) {
		//获取入参
		String reqdata=reqModel.getData();
		JSONArray dataArray = JSONObject.parseArray(reqdata);

		//获取系统现有部门数据
		QFilter numFilter = new QFilter("structure.view.number", QCP.equals,"01");
		DynamicObjectCollection bosAdminorg = QueryServiceHelper.query("bos_adminorg", "number,yrpg_mainid,structure.isfreeze", new QFilter[]{numFilter});
		Map<String,Boolean> adminStatusMap = new HashMap<String,Boolean>();
		Map<String,String> adminNumMap = new HashMap<String,String>();
		for (DynamicObject admin : bosAdminorg){
			adminStatusMap.put(admin.getString("yrpg_mainid"),admin.getBoolean("structure.isfreeze"));
			adminNumMap.put(admin.getString("number"),admin.getString("yrpg_mainid"));
		}
		//参数重新组装成 新增（修改）、启用（解封）、禁用（封存）参数
		JSONArray adminData = new JSONArray();
		JSONArray freezeNums = new JSONArray();
		JSONArray unfreezeNums = new JSONArray();

		JSONObject saveDataJson = new JSONObject();
		JSONObject freezeDataJson = new JSONObject();
		JSONObject unfreezeDataJson = new JSONObject();

		JSONObject freezeNumsJson = new JSONObject();
		JSONObject unfreezeNumsJson = new JSONObject();
		//循环判断部门进入保存、封存、解封集合
		for(int i=0;i<dataArray.size();i++){
			JSONObject jsonObject = dataArray.getJSONObject(i);
			String number = jsonObject.getString("yrpg_mainid");
			boolean isfreeze = jsonObject.getBoolean("isfreeze");
			//根据入参是否封存，已经系统中状态判断
			if(adminStatusMap.get(number)==null){
				adminData.add(jsonObject);
			}else if(!isfreeze&&!adminStatusMap.get(number)){
				adminData.add(jsonObject);
			}else if(!adminStatusMap.get(number)&&isfreeze){
				freezeNums.add(number);
			}else if(adminStatusMap.get(number)&&!isfreeze){
				unfreezeNums.add(number);
			}
		}

		//返回值
		BaseFilterResult responseData = new BaseFilterResult();
		CustomApiResult finalResult =new CustomApiResult();
		//判断是否调用封存解封
		boolean isfreeze = false;
		//调用封存接口
		if(freezeNums.size()>0){
			isfreeze = true;
			freezeNumsJson.put("yrpg_mainids",freezeNums);
			freezeDataJson.put("data",freezeNumsJson);
			//调用接口
			OpenApiResult adminfreeze = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminfreeze", freezeDataJson, new HashMap<>());
			BaseFilterResult data = (BaseFilterResult)adminfreeze.getData();
			List<BaseFilterItemData> result = data.getResult();
			for(BaseFilterItemData item : result){
				String number = item.getNumber();
				if(!StringUtils.isEmpty(number)){
					item.setNumber(adminNumMap.get(number));
				}
			}
			responseData = data;
		}
		//调用解封接口
		if(unfreezeNums.size()>0){
			isfreeze = true;
			unfreezeNumsJson.put("yrpg_mainids",unfreezeNums);
			unfreezeDataJson.put("data",unfreezeNumsJson);
			//调用接口
			OpenApiResult adminunfreeze = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminunfreeze", unfreezeDataJson, new HashMap<>());
			BaseFilterResult data = (BaseFilterResult)adminunfreeze.getData();
			List<BaseFilterItemData> result = data.getResult();
			for(BaseFilterItemData item : result){
				String number = item.getNumber();
				if(StringUtils.isEmpty(number)){
					item.setNumber(adminNumMap.get(number));
				}
			}
			if(responseData==null){
				responseData = data;
			}else{
				BaseFilterResult basefilterResult = (BaseFilterResult)responseData;
				long failCount = data.getFailCount();
				long successCount = data.getSuccessCount();
				long totalCount = data.getTotalCount();
				//接口重新合计
				basefilterResult.setFailCount(failCount+basefilterResult.getFailCount());
				basefilterResult.setSuccessCount(basefilterResult.getSuccessCount()+successCount);
				basefilterResult.setTotalCount(basefilterResult.getTotalCount()+totalCount);
				basefilterResult.getResult().addAll(data.getResult());
			}
		}
		//调用保存接口
		if(adminData.size()>0){
			saveDataJson.put("data",adminData);
			//调用接口
			OpenApiResult adminsave = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminsave", saveDataJson, new HashMap<>());
			ApiSaveResult data = (ApiSaveResult)adminsave.getData();
			//根据查询更新操作返回值结构，将保存返回值组装
			List<ApiSaveItemData> result = data.getResult();
			for(ApiSaveItemData item:result){
				BaseFilterItemData baseFilterItemData = new BaseFilterItemData();
				baseFilterItemData.setNumber(item.getNumber());
				//报错时重新设置编码
				baseFilterItemData.setNumber(item.getKeys().get("yrpg_mainid"));
//				if(StringUtils.isEmpty(baseFilterItemData.getNumber())){
//				}
				baseFilterItemData.setId(item.getId());
				//组装错误信息
				List<ApiSaveRowErrorData> errors = item.getErrors();
				List<String> curerrors = new ArrayList<String>();
				for(ApiSaveRowErrorData errer:errors){
					StringBuffer sb = new StringBuffer();
					Set<String> rowMsg = errer.getRowMsg();
					for(String err:rowMsg){
						sb = sb.append(err);
					}
					curerrors.add(sb.toString());
				}
				baseFilterItemData.setErrors(curerrors);
				//根据错误信息判断状态
				if(errors.size()>0){
					baseFilterItemData.setBillStatus(false);
				}else{
					baseFilterItemData.setBillStatus(true);
				}
				responseData.getResult().add(baseFilterItemData);
			}
			long failCount = data.getFailCount();
			long successCount = data.getSuccessCount();
			//重新设置合计数
			if(isfreeze){
				responseData.setTotalCount(responseData.getTotalCount()+failCount+successCount);
				responseData.setSuccessCount(responseData.getSuccessCount()+successCount);
				responseData.setFailCount(responseData.getFailCount()+failCount);
			}else{
				responseData.setTotalCount(failCount+successCount);
				responseData.setSuccessCount(successCount);
				responseData.setFailCount(failCount);
			}
		}
		//多类型接口 过滤默认为空
		responseData.setFilter("");
		finalResult.setData(responseData);
		return finalResult;
	}


}